/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Jianfeng Gao ( jianfeng dot gao at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include <SimoxUtility/json.h>
#include <RobotAPI/libraries/core/Pose.h>
#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/ManagedIceObject.h>

#include <Eigen/Geometry>
#include <mplib/core/types.h>



namespace armarx::control::common
{

Eigen::VectorXf getEigenVec(nlohmann::json& userConfig, nlohmann::json& defaultConfig, const std::string& entryName, int size = 0, int value = 0);

template <typename T>
void getEigenVec(nlohmann::json& userConfig, nlohmann::json& defaultConfig, const std::string& entryName, T& vec)
{
    if (userConfig.find(entryName) != userConfig.end())
    {
        vec = userConfig[entryName];
    }
    else
    {
        vec = defaultConfig[entryName];
    }
}

void getEigenMatrix(nlohmann::json& userConfig, const std::string& entryName, Eigen::MatrixXf& mat);

Eigen::VectorXf getEigenVecWithDefault(nlohmann::json& userConfig, Eigen::VectorXf defaultValue, const std::string& entryName);

template <typename T>
T getValue(nlohmann::json& userConfig, nlohmann::json& defaultConfig, const std::string& entryName)
{
    if (userConfig.find(entryName) != userConfig.end())
    {
        return userConfig[entryName].get<T>();
    }
    else
    {
        return defaultConfig[entryName].get<T>();
    }
}

template <typename T>
T getValueWithDefault(nlohmann::json& userConfig, T defaultValue, const std::string& entryName)
{
    if (userConfig.find(entryName) != userConfig.end())
    {
        return userConfig[entryName].get<T>();
    }
    else
    {
        return defaultValue;
    }
}

void debugEigenPose(StringVariantBaseMap& datafields, const std::string& name, Eigen::Matrix4f pose);
void debugEigenVec(StringVariantBaseMap& datafields, const std::string& name, Eigen::VectorXf vec);

void getError(Eigen::Matrix4f& currentPose, Eigen::Vector3d& targetPosition, Eigen::Quaterniond& targetQuaternion, double& posiError, double& oriError);

void to_json(nlohmann::json& j, const Pose& fp);

void from_json(const nlohmann::json& j, Pose& fp);

void to_json(nlohmann::json& j, const PosePtr& fp);

void from_json(const nlohmann::json& j, PosePtr& fp);

void to_json(nlohmann::json& j, const PoseBasePtr& fp);

void from_json(const nlohmann::json& j, PoseBasePtr& fp);

PosePtr dVecToPose(const mplib::core::DVec& dvec);

Eigen::Matrix4f dVecToMat4(const mplib::core::DVec& dvec);

mplib::core::DVec poseToDVec(PosePtr& pose);

mplib::core::DVec mat4ToDVec(Eigen::Matrix4f& mat);

std::vector<float> poseToFVec(PosePtr& pose);

std::vector<float> mat4ToFVec(Eigen::Matrix4f& mat);

std::string dVecToString(const mplib::core::DVec& dvec);

Eigen::VectorXf vecToEigen(std::vector<float> &vec);
}
