#include "Component.h"

#include <ArmarXCore/libraries/DecoupledSingleComponent/Decoupled.h>

#include <armarx/control/skills/skills/MoveJointsToPosition.h>
#include <armarx/control/skills/skills/MoveJointsWithVelocity.h>
#include <armarx/control/skills/skills/RelaxHand.h>
#include <armarx/control/skills/skills/ShapeHand.h>

namespace armarx::control::components::control_skill_provider
{

    ::armarx::PropertyDefinitionsPtr
    Component::createPropertyDefinitions()
    {
        ::armarx::PropertyDefinitionsPtr def =
            new ::armarx::ComponentPropertyDefinitions(getConfigIdentifier());

        def->component(remote.kinematicUnit);
        def->component(remote.leftHandUnit, "LeftHandUnit", "LeftHandUnit");
        def->component(remote.rightHandUnit, "RightHandUnit", "RightHandUnit");

        def->required(properties.robotName, "RobotName", "Name of the robot.");

        return def;
    }

    void
    Component::onInitComponent()
    {
    }

    void
    Component::onConnectComponent()
    {
        armarx::armem::robot_state::VirtualRobotReader robotStateReader;
        robotStateReader.connect(memoryNameSystem());

        {
            {
                armarx::control::skills::skills::MoveJointsToPosition::Remote skillRemote{
                    {remote.kinematicUnit}, robotStateReader, properties.robotName};
                addSkillFactory<armarx::control::skills::skills::MoveJointsToPosition>(skillRemote);
            }
            {
                armarx::control::skills::KinematicUnitRemote skillRemote{remote.kinematicUnit};
                addSkillFactory<armarx::control::skills::skills::MoveJointsWithVelocity>(
                    skillRemote);
            }
            {
                armarx::control::skills::HandUnitRemote skillRemote{remote.leftHandUnit,
                                                                    remote.rightHandUnit};
                addSkillFactory<armarx::control::skills::skills::ShapeHand>(skillRemote);
            }
            {
                addSkillFactory<armarx::control::skills::skills::SimpleShapeHand>("Open");
            }
            {
                addSkillFactory<armarx::control::skills::skills::SimpleShapeHand>("Close");
            }
            {
                armarx::control::skills::HandUnitRemote skillRemote{remote.leftHandUnit,
                                                                    remote.rightHandUnit};
                addSkillFactory<armarx::control::skills::skills::RelaxHand>(skillRemote);
            }
        }
    }

    void
    Component::onDisconnectComponent()
    {
    }

    void
    Component::onExitComponent()
    {
    }

    const std::string Component::defaultName = "control_skill_provider";

    std::string
    Component::getDefaultName() const
    {
        return Component::defaultName;
    }

    std::string
    Component::GetDefaultName()
    {
        return Component::defaultName;
    }

    ARMARX_REGISTER_COMPONENT_EXECUTABLE(Component, Component::GetDefaultName());

} // namespace armarx::control::components::control_skill_provider
