/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * package    control::controller_creator
 * author     Jianfeng Gao ( jianfeng dot gao at kit dot edu )
 * date       2023
 * copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *            GNU General Public License
 */


#pragma once

#include <RobotAPI/interface/armem/client/MemoryListenerInterface.ice>

module armarx {  module control {  module components {  module controller_creator
{

    interface ComponentInterface extends armarx::armem::client::MemoryListenerInterface
    {
        string createController(string controllerNamePrefix, string robotNodeSet, string controllerType, string configFilename);
        bool updateTargetPose(string controllerName, string controllerType, Ice::FloatSeq targetPoseVector);
        bool updateTargetNullspaceJointAngles(string controllerName, string controllerType, Ice::FloatSeq targetNullspaceJointAngle);
        bool updateTarget(string controllerName, string controllerType, Ice::FloatSeq targetPoseVector, Ice::FloatSeq targetNullspaceJointAngle);
        string kinestheticTeaching(string kinematicChainName, string side, string fileName);
        void enableVelocityMode();

        Ice::FloatSeq getPoseInRootFrame(string node_name);
        Ice::FloatSeq getPrevTargetPose(string controllerName);
        Ice::FloatSeq getPrevNullspaceTargetAngle(string controllerName);
    };

};};};};
