#include "RetrieveHand.h"

#include <armarx/control/retrieve_hand/constants/constants.h>
#include <armarx/control/deprecated_njoint_mp_controller/task_space/ControllerInterface.h>

namespace armarx::control::retrieve_hand::skills
{

    ::armarx::skills::SkillDescription
    RetrieveHand::GetSkillDescription()
    {
        ParamType defaultParams;
        // defaultParams.exampleStringParam = "exampleDefaultValue";

        auto skillId = armarx::skills::SkillID{.skillName = armarx::control::retrieve_hand::constants::SKILL_NAME};

        return ::armarx::skills::SkillDescription{
            .skillId = skillId,
            .description = "TODO: Description of skill RetrieveHand.",
            .rootProfileDefaults = defaultParams.toAron(),
            .timeout = ::armarx::Duration::MilliSeconds(1000),
            .parametersType = ParamType::ToAronType()};
    }

    RetrieveHand::RetrieveHand(const Remote& r) :
            Base(GetSkillDescription()), remote(r)

    {
    }


    RetrieveHand::RetrieveHand() :
        Base(GetSkillDescription())
    {
    }

    /*
    ::armarx::skills::Skill::InitResult
    RetrieveHand::init(const SpecializedInitInput&)
    {
        return InitResult{.status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    }
    */


    ::armarx::skills::Skill::MainResult
    RetrieveHand::main(const SpecializedMainInput& in)
    {
        // Enter main code of the skill here.
        core::RetrieveHand::Remote rem{
                .memoryNameSystem = remote.memoryNameSystem,
                .robotUnit = remote.robotUnit};

        core::RetrieveHand::Properties props{
                .robotName = in.parameters.robotName,
                .robotNodeSet = in.parameters.robotNodeSet,
                .timeDurationInSec = in.parameters.timeDurationInSec,
                .targetPose = in.parameters.targetPose,
                .kpLinear = in.parameters.kpLinear,
                .kpAngular = in.parameters.kpAngular,
                .kdLinear = in.parameters.kdLinear,
                .kdAngular = in.parameters.kdAngular,
                .fileNames = in.parameters.fileNames,
        };


        impl = std::make_unique<core::RetrieveHand>(rem, props);

        auto res = impl->execute();
        if (res)
        {
            return MakeSucceededResult();
        }
        return MakeFailedResult();
    }

    /*
    ::armarx::skills::Skill::ExitResult
    RetrieveHand::exit(const SpecializedExitInput&)
    {
        return ExitResult{.status = ::armarx::skills::TerminatedSkillStatus::Succeeded};
    }
    */

}
