#pragma once

#include <RobotAPI/interface/units/KinematicUnitInterface.h>
#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem_robot_state/client/common/VirtualRobotReader.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/control/skills/skills/aron/MoveJointsWithVelocityParams.aron.generated.h>

#include "../skills.h"

namespace armarx::control::skills::skills
{

    /**
   * @class MoveJoints
   * @brief MoveJoints-skill to move joints to specified target values using the KinematicUnit
   *
   * Sets joint values with the KinematicUnit and waits until target values are reached. Can
   * simulate trajectory when in simulation (otherwise trajectories from joint value updates through
   * the KinematicUnit are not simulated).
   */
    class MoveJointsWithVelocity :
        public ::armarx::skills::SimpleSpecializedSkill<MoveJointsWithVelocityParams>
    {

    public:
        static ::armarx::skills::SkillDescription
        GetSkillDescription()
        {
            return ::armarx::skills::SkillDescription{
                .skillId = {.skillName = "MoveJointsWithVelocity"},
                .description = "Moves joints with a specific velocity using the KinematicUnit",
                .timeout = ::armarx::Duration::Minutes(1),
                .parametersType = ParamType::ToAronType()};
        }

        MoveJointsWithVelocity(const KinematicUnitRemote&);


    private:
        ::armarx::skills::Skill::MainResult main(const SpecializedMainInput& in) override;
        ::armarx::skills::Skill::ExitResult exit(const SpecializedExitInput& in) override;

        std::map<std::string, float>
        filterJointTargetValues(const NameValueMap& jointsTargetValues,
                                const std::vector<std::string>& disabledJoints) const;

        void switchControlMode(const NameValueMap& jointsTargetValues,
                               const armarx::ControlMode controlMode);

        void moveJoints(const NameValueMap& jointsTargetValues,
                        const armarx::core::time::Duration& dur);

    private:
        KinematicUnitRemote remote;
    };

} // namespace armarx::control::skills::skills
